%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% RUNME.m
% by Ana de Sousa (anacsousa@lara.unb.br)
% and Antonio Padilha L. Bo (antonio.plb@lara.unb.br)
% Last update: July 2018
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
tic; clear; close all; clc;
import org.opensim.modeling.*
addpath('OpensimLibrary');

%% GLOBAL VARIABLES
% timing
global Tf
global Ts
global n_samples

% general variables
global ZERO
global refVel
global measAngle
global measVel
global musclesNames
global statesNames

% general control variables
global controlType
global controlAction
global controlActionQuadRight
global controlActionQuadLeft
global controlActionHamsRight
global controlActionHamsLeft
global controlActionGlutRight
global controlActionGlutLeft
global controlError
global controlTime
global controlIteration
global integralError

% flags
global flagSensorNoise
global flagCorrection

%% INITIAL CONFIGURATION
% ---> choose final time, the control sample time
Tf = 2.0; Ts = .02; n_samples = Tf/Ts;

% ---> Adds a noisy signal to the inertial sensor
flagSensorNoise = 1;

% ---> Implements the speed correction (Cadence-based Phase Adjustment)
flagCorrection = 1;

% ---> choose the controller (UNCOMMENT THE DESIRED CONTROLLER)
controlType = 'PID';
% controlType = 'fuzzy';
% controlType = 'Open-loop';

% do not change:
ZERO = 1e-3;
refVel = zeros(1,n_samples);
measAngle = zeros(1,n_samples);
measVel = zeros(1,n_samples);
controlAction = ZERO * ones(1,n_samples);
controlActionQuadRight = ZERO * ones(1,n_samples);
controlActionQuadLeft = ZERO * ones(1,n_samples);
controlActionHamsRight = ZERO * ones(1,n_samples);
controlActionHamsLeft = ZERO * ones(1,n_samples);
controlActionGlutRight = ZERO * ones(1,n_samples);
controlActionGlutLeft = ZERO * ones(1,n_samples);
controlError = zeros(1,n_samples);
controlTime = zeros(1,n_samples);
controlIteration = 1;
integralError = 0;

%% INITIALIZE MODEL
disp('> Initialize simulation')
% open and init model
cyclingModel = Model('Model/FESCyclingModel.osim');
cyclingStates = cyclingModel.initSystem();
cyclingModel.equilibrateMuscles(cyclingStates);

% control function handle
controlFunctionHandle = @cycling_control;

% get muscles and states names
musclesNames = get_muscles_names(cyclingModel);
statesNames = get_states_names(cyclingModel);

%% CONFIG & RUN SIMULATION
disp('> Run simulation')

% integrate plant using Matlab integration
timeSpan = [0 Tf]; integratorName = 'ode15s'; 
integratorOptions = odeset('AbsTol', 1E-5','MaxStep',.1*Ts);

% run simulation using function from Dynamic Walking example
motionData = IntegrateOpenSimPlant(cyclingModel, controlFunctionHandle, timeSpan, ...
    integratorName, integratorOptions);

%% SHOW RESULTS
n_samples = controlIteration;

refVel = refVel(1:n_samples);
measAngle = measAngle(1:n_samples);
measVel = measVel(1:n_samples);
controlActionQuadRight = controlActionQuadRight(1:n_samples);
controlActionQuadLeft = controlActionQuadLeft(1:n_samples);
controlActionHamsRight = controlActionHamsRight(1:n_samples);
controlActionHamsLeft = controlActionHamsLeft(1:n_samples);
controlActionGlutRight = controlActionGlutRight(1:n_samples);
controlActionGlutLeft = controlActionGlutLeft(1:n_samples);
controlError = controlError(1:n_samples);
controlTime = controlTime(1:n_samples);

figure();
subplot(5,1,1); hold on;
plot(controlTime,controlActionQuadRight,'b');
plot(controlTime,controlActionQuadLeft,'r'); hold off;
xlabel('time [s]'); ylabel('Control Action');  ylim([0,1.1]);

subplot(5,1,2); hold on;
plot(controlTime,controlActionHamsRight,'b');
plot(controlTime,controlActionHamsLeft,'r'); hold off;
xlabel('time [s]'); ylabel('Control Action');  ylim([0,1.1]);

subplot(5,1,3); hold on;
plot(controlTime,controlActionGlutRight,'b');
plot(controlTime,controlActionGlutLeft,'r'); hold off;
xlabel('time [s]'); ylabel('Control Action');  ylim([0,1.1]);

subplot(5,1,4);
plot(controlTime,wrapTo360(rad2deg(-measAngle)),'b');
xlabel('time [s]'); ylabel('Gear Angle [deg]'); ylim([-10,370]);

subplot(5,1,5); hold on;
plot(controlTime,rad2deg(measVel),'b');
plot(controlTime, rad2deg(refVel), 'r'); hold off;
xlabel('time [s]'); ylabel('Angular speed [deg/s]'); ylim([-30,400]);

%% CREATE .STO FILE FOR VISUALIZATION, SAVE WORKSPACE AND SAVE FIGURE
Nsamples = length(motionData.data(:,1)); Nstates = length(statesNames);

str = strjoin(statesNames,'\t');
header = ['cycling_simulation \nversion=1 \nnRows=' num2str(Nsamples) ' \nnColumns=' num2str(Nstates+1) '\ninDegrees=no \nendheader \ntime	' str '\n'];
fid = fopen('Results/cycling_simulation.sto','wt');
fprintf(fid,header); fclose(fid);
fid = fopen('Results/cycling_simulation.sto','a+');
for i = 1:Nsamples
    fprintf(fid,'\t%f',motionData.data(i,:)); fprintf(fid,'\n');
end
fclose(fid);

varlist = {'cyclingModel','cyclingStates'}; clear(varlist{:}); % clear opensim var
save('Results\cycling_simulation.mat');                   % save data
savefig('Results\cycling_simulation.fig');                % save fig


%% The end!
fprintf('\n'); toc; disp('> THE END')
