function modelControls = cycling_control(osimModel, osimState)
    import org.opensim.modeling.*;

    %% GLOBAL VARIABLES
    % timing
    global Ts
    global n_samples

    % general variables
    global ZERO
    global refVel
    global measAngle
    global measVel
    global musclesNames

    % general control variables
    global controlType

    global controlAction
    global controlActionQuadRight
    global controlActionQuadLeft
	global controlActionHamsRight
	global controlActionHamsLeft
	global controlActionGlutRight
	global controlActionGlutLeft

    global controlError
    global controlTime
    global controlIteration

    global integralError

    % flags
    global flagSensorNoise
    global flagCorrection

    %% GET INFO
    modelControls = osimModel.updControls(osimState);
    thisTime = osimState.getTime();
    thisStateArray = osimModel.getStateValues(osimState);
    thisAngle = thisStateArray.get(10);
    thisVel = -(thisStateArray.get(23));

    %% UPDATE CONTROL (only update if sampling period has passed)
    if (thisTime - controlTime(controlIteration)) >= (Ts-.01*Ts)
        
        % add noise
        if flagSensorNoise
            sigma = deg2rad(5);
            thisAngle = thisAngle + sigma*randn(1);
            thisVel = thisVel + sigma*randn(1);
        end
        
        controlIteration = controlIteration + 1;
        controlTime(controlIteration) = thisTime;

        % print simulation evolution        
        fprintf('%d/%d\n',controlIteration, n_samples);

        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % CONTROLLER CODE HERE
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

        % define reference
        thisRefVel = speed_step_sequence(controlIteration);

        % compute error 
        thisError = thisRefVel - thisVel;

        % compute control
        if strcmp(controlType,'ILC')
            if controlIteration >= 350 % ILC PD-type
                % thisU = pedal_ilc_pdtype( thisRefPedal, thismeasAngle );
                % ILC PD-type with Q-filter
                thisU = pedal_ilc_pdtype_qfilter( thisRefVel, thisAngle );
            else % do PID
                [thisU, integralError] = cycling_pid(thisRefVel, thisAngle, controlAction(controlIteration-1), ...
                    controlError(controlIteration-1), integralError, Ts);
            end
        elseif strcmp(controlType,'PID')
            [thisU, integralError] = cycling_pid(thisRefVel, thisVel, controlAction(controlIteration-1), ...
                controlError(controlIteration-1), integralError, Ts);
            if thisU < 0
                thisU  = 0; % turn off stimulation if speed is higher
            end
        elseif strcmp(controlType, 'fuzzy')
            [thisU] = cycling_fuzzy(thisRefVel, thisVel, controlError(controlIteration-1), ...
                 Ts);
            if thisU < 0
                thisU  = 0; % turn off stimulation if speed is higher
            end 
        elseif strcmp(controlType,'Open-loop')
            if controlIteration <= 0.1/Ts % start open-loop step excitation at this instant
                thisU = 0;
            else
                thisU = cycling_openloop( thisTime );
            end
        end
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % MUSCLE RANGE
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

        % define muscle actuation
        if flagCorrection
            [ quad_l, quad_r , hams_l, hams_r, glut_l, glut_r ] = profile( thisAngle,  thisVel , 1);
        else
            [ quad_l, quad_r , hams_l, hams_r, glut_l, glut_r ] = profile( thisAngle,  thisVel , 0 );
        end
        quadRight = thisU*quad_r;
        quadLeft = thisU*quad_l;
        hamsRight = thisU*hams_r;
        hamsLeft = thisU*hams_l;
        glutRight = thisU*glut_r;
        glutLeft = thisU*glut_l;
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % END OF CONTROLLER CODE
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

        refVel(controlIteration) = thisRefVel;
        controlAction(controlIteration) = thisU;
        controlError(controlIteration) = thisError;
        measAngle(controlIteration) = thisAngle;
        measVel(controlIteration) = thisVel;
        
        controlActionQuadRight(controlIteration) = quadRight;
        controlActionQuadLeft(controlIteration) = quadLeft;
        controlActionHamsRight(controlIteration) = hamsRight;
        controlActionHamsLeft(controlIteration) = hamsLeft;
        controlActionGlutRight(controlIteration) = glutRight;
        controlActionGlutLeft(controlIteration) = glutLeft;
                
    else
        thisU = controlAction(controlIteration);
        
        quad_r = controlActionQuadRight(controlIteration);
        quad_l = controlActionQuadLeft(controlIteration);
        hams_r = controlActionHamsRight(controlIteration);
        hams_l = controlActionHamsLeft(controlIteration);
        glut_r = controlActionGlutRight(controlIteration);
        glut_l = controlActionGlutLeft(controlIteration);
        
        quadRight = thisU*quad_r;
        quadLeft = thisU*quad_l;
        hamsRight = thisU*hams_r;
        hamsLeft = thisU*hams_l;
        glutRight = thisU*glut_r;
        glutLeft = thisU*glut_l;
    end
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % END OF MUSCLE ENCODING
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    % assing excitation to muscles
    for i = 1:length(musclesNames)
        if strcmp(musclesNames{i},'rect_fem_r')
            thisExcitation = Vector(1, quadRight);
        elseif strcmp(musclesNames{i},'vas_int_r')
            thisExcitation = Vector(1, quadRight);
        elseif strcmp(musclesNames{i},'rect_fem_l')
            thisExcitation = Vector(1, quadLeft);
        elseif strcmp(musclesNames{i},'vas_int_l')
            thisExcitation = Vector(1, quadLeft);
		elseif strcmp(musclesNames{i},'hamstrings_l')
            thisExcitation = Vector(1, hamsLeft);		
		elseif strcmp(musclesNames{i},'hamstrings_r')
            thisExcitation = Vector(1, hamsRight);
		elseif strcmp(musclesNames{i},'glut_max_r')
            thisExcitation = Vector(1, glutRight);		
		elseif strcmp(musclesNames{i},'glut_max_l')
            thisExcitation = Vector(1, glutLeft);
        else
            thisExcitation = Vector(1, ZERO);
        end
        % update modelControls with the new values
        if thisExcitation ~= ZERO
            osimModel.updActuators().get(musclesNames{i}).addInControls(thisExcitation, modelControls);
        end
    end
    
end